<?php
namespace Quivers\Integration\Model;

use Stripe\Stripe;
use \Magento\Customer\Model\Session as customerSession;
use Quivers\Integration\LogTracking;

class QuiversStripe extends \Magento\Payment\Model\Method\Cc
{

    protected $_code = 'quivers_integration';
    protected $_canAuthorize = true;
    protected $_isGateway = true;
    protected $curlClient;
    protected $_stripeApi = false;
    protected $_scopeConfig;


    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory,
        \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory,
        \Magento\Payment\Helper\Data $paymentData,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Payment\Model\Method\Logger $logger,
        \Magento\Framework\Module\ModuleListInterface $moduleList,
        \Magento\Framework\Stdlib\DateTime\TimezoneInterface $localeDate,
        \Magento\Framework\Message\ManagerInterface $messageManager,
        customerSession $customerSession
    ) {
        parent::__construct(
            $context,
            $registry,
            $extensionFactory,
            $customAttributeFactory,
            $paymentData,
            $scopeConfig,
            $logger,
            $moduleList,
            $localeDate
        );
        $this->_scopeConfig = $scopeConfig;
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $this->curlClient = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\HTTP\Client\Curl::class);
        $this->_logger = $logger;
    
       
        // get the configured values in payment configuration form.
        $secret_key = $this->_scopeConfig->getValue("payment/quivers_integration/secret_key", $storeScope);
        $this->_minAmount = $this->_scopeConfig->getValue("payment/quivers_integration/min_order_total", $storeScope);
        $this->customerSession = $customerSession;
        $this->messageManager = $messageManager;
        Stripe::setApiKey($secret_key);
    }

    public function assignData(\Magento\Framework\DataObject $data)
    {
        parent::assignData($data);
        $additionalData = $data->getAdditionalData();
        if (is_array($additionalData))
            $data->setData(array_merge($data->getData(), $additionalData));

        $info = $this->getInfoInstance();
        
        $token = $data['cc_stripejs_token'];
        $saveMyCard = $data['cc_save_card'];
        $captchatoken = $data['captcha_token'];
        // assigning data received from payment form so that it can be used
        // later in authorize method.
        $info->setAdditionalInformation('cc_stripejs_token', $token);
        $info->setAdditionalInformation('captcha_token', $captchatoken);
        $info->setAdditionalInformation('cc_save_card', $saveMyCard);
        return $this;
    }

   
    public function authorize(\Magento\Payment\Model\InfoInterface $payment, $amount)
    {
        // get saved data
        $startTime = microtime(true);
        $log = new LogTracking($this->_scopeConfig);
        $payment_token = $payment->getAdditionalInformation('cc_stripejs_token');
        $saveMyCard = $payment->getAdditionalInformation('cc_save_card');
        $token = $payment->getAdditionalInformation('captcha_token');
        $order = $payment->getOrder();
        $billing = $order->getBillingAddress();
        $customerEmail = $billing->getEmail();

        try {
            // check if user with same email already exists in Stripe account.
            $customer = false;
            $customersList = \Stripe\Customer::all(array(
                "limit" => 1,
                "email" => $customerEmail,
            ));

            // create a new user if it doesn't exists.
            if(count($customersList["data"])>0) {
                $customer = $customersList["data"][0];
            }

            if(!$customer){
                $customer = \Stripe\Customer::create(array(
                'source' => $payment_token,
                'email' => $customerEmail,
                ));
            }

            $customerId = $customer["id"];

            if ($saveMyCard)
            {  // checked
               try {
                   $customer->sources->create(array("source" => $payment_token));
                   }
               catch(\Exception $e){
                    }
            }
               $customerId = $customer["id"];

            $customer = $this->customerSession->getCustomer();
            if(!$customer){
                throw new \Magento\Framework\Validator\Exception(__(' Unable to Fetch Customer Details'));
            }
            $myCustomerId = $customer->getId();
            if(isset($customerId) && $myCustomerId) {
                $customer->setAttributeSetId(\Magento\Customer\Api\CustomerMetadataInterface::ATTRIBUTE_SET_ID_CUSTOMER);
                $customer->setQuiversStripeCustomerId($customerId);
                if ($saveMyCard) {  // checked
                    // add current card.
                    $savedCardIds = explode(',', $customer->getQuiversStripeCardIds());
                    if(count($savedCardIds)==10){
                        unset($savedCardIds[1]);
                    }
                    $savedCardIds[] = $payment_token;  
                    $customer->setQuiversStripeCardIds(implode(",", $savedCardIds));
                }
                $customer->save();
            }
        } catch (\Exception $e) {
            throw new \Magento\Framework\Validator\Exception(__($e->getMessage().' Please verify the details before trying again. If the issue persists please contact Quivers at 805-250-4555'));
        }
 
        $secret = isset($_SESSION['site_keys']['site_key_secret'])?$_SESSION['site_keys']['site_key_secret']:'NO-SECRET_KEY';

        if(isset($token) && $token!=null && isset($secret)):
            //get verify response data
            $verifyResponse = file_get_contents('https://www.google.com/recaptcha/api/siteverify?secret='.$secret.'&response='.$token.'&remoteip='.$_SERVER["REMOTE_ADDR"]);
            $responseData = json_decode($verifyResponse);
            $responseData = json_decode($verifyResponse);
            $log->captcha_data($token,$responseData->success);
                if($responseData->success):
                    // $charge = $this->createCharge($customerId, $payment, $amount);
                else:
                    $responseData = json_decode(json_encode($responseData),true);
                    if(isset($responseData["error-codes"][0]) && $responseData["error-codes"][0]=='invalid-input-secret') :
                        // $charge = $this->createCharge($customerId, $payment, $amount);
                    else:
                        $message = "Please reload your page  and try again. If the issue still persists,please contact 'enterprise@quivers.com' for further assistance";
                        throw new \Magento\Framework\Validator\Exception(__($message));
                    endif;
                endif;
        endif;
        try
        {
            $payment_intent_id=$this->customerSession->getPaymentIntentId();
            #retieve payment intent to get charge id and status
            $payment_intent=\Stripe\PaymentIntent::retrieve($payment_intent_id);
            #set get charge id and status to payemnt object
            $payment
                ->setTransactionId($payment_intent->charges->data[0]->id)
                ->setIsTransactionClosed(0);
            #sending logs
            $order_data = ["Customer Id"=>$order->getCustomerId(), "Email"=>$order->getCustomerEmail()];
            $response = ["Response"=>$payment_intent->charges->data[0], "Amount"=>$amount];
            $log->final_payment_intent("stripe", date('r', $startTime), $payment_intent);
            $log->payment_request($order_data, "stripe", date('r', $startTime), $response, $payment_intent->status);
            $this->customerSession->unsetPaymentIntentId();
            $this->customerSession->unsetTotalAmount();
            return $this;
        }
        catch(\Exception $e)
        {   
            throw new \Magento\Framework\Validator\Exception(__("Sorry! Looks like the Payment Method is not yet available to use because it has not yet been Fully Configured."));
        }
    }
    
    public function formatCurrencyAmount($amount, $currency)
    {

        // Stripe takes amounts as the smallest currency unit. ...

        if (empty($amount)) {
            return null;
        }
        $multiple = 100;
        if (in_array(
            $currency,
            array('BIF', 'DJF', 'JPY', 'KRW', 'PYG', 'VND', 'XAF', 'XPF', 'CLP', 'GNF', 'KMF', 'MGA', 'RWF', 'VUV', 'XOF'),
            true
        )) {
            $multiple = 1;
        }

        return sprintf('%01.0f', (float)$amount * $multiple);
    }



    public function isAvailable(\Magento\Quote\Api\Data\CartInterface $quote = null)
    {
        if ($quote && $quote->getBaseGrandTotal() <= $this->_minAmount) {
            return false;
        }
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $secretKey = $this->_scopeConfig->getValue("payment/quivers_integration/secret_key", $storeScope);
        $activekey = $this->_scopeConfig->getValue("payment/quivers_integration/active", $storeScope);
        $active = $this->_scopeConfig->getValue("quivers/parameters/active", $storeScope);
        if (!$secretKey  or $active==0) {
            return false;
        }

        return $activekey;
    }
    public function canUseForCurrency($currencyCode)
    {
        return true;
    }

    public function validate()
    {
        // validation is already handled by stripejs
        return true;
    }

}
