<?php
namespace Quivers\Integration\Setup;

use Magento\Eav\Setup\EavSetup;
use Magento\Eav\Setup\EavSetupFactory;
use Magento\Framework\Setup\UpgradeDataInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\ModuleDataSetupInterface;

class UpgradeData implements UpgradeDataInterface
{
	private $eavSetupFactory;

	public function __construct(
		EavSetupFactory $eavSetupFactory, 
		\Magento\Customer\Setup\CustomerSetupFactory $customerSetupFactory,
        \Magento\Eav\Api\AttributeRepositoryInterface $attributeRepository
    )
	{
		$this->eavSetupFactory = $eavSetupFactory;
		$this->customerSetupFactory = $customerSetupFactory;
        $this->attributeRepository = $attributeRepository;
	}
	
	public function upgrade(ModuleDataSetupInterface $setup, ModuleContextInterface $context)
    {
		if (version_compare($context->getVersion(), '1.7.2') <= 0) {
        
	        $eavSetup = $this->eavSetupFactory->create(['setup' => $setup]);

	        $eavSetup->addAttribute(
				\Magento\Catalog\Model\Product::ENTITY,
				'product_tax_category',
				[
					'type' => 'text',
					'backend' => '',
					'frontend' => '',
					'label' => 'Product Tax Code',
					'input' => 'text',
					'class' => '',
					'source' => '',
					'global' => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
					'visible' => true,
					'required' => false,
					'sort_order' => 900,
					'user_defined' => false,
					'default' => '',
					'searchable' => false,
					'filterable' => false,
					'comparable' => false,
					'visible_on_front' => false,
					'used_in_product_listing' => true,
					'unique' => false,
					'apply_to' => ''
				]
			);
			$eavSetup->addAttribute(
				\Magento\Catalog\Model\Product::ENTITY,
				'quivers_claiming_group',
				[
					'type' => 'text',
					'backend' => '',
					'frontend' => '',
					'label' => 'Quivers Claiming Group',
					'input' => 'text',
					'class' => '',
					'source' => '',
					'global' => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
					'visible' => true,
					'required' => false,
					'sort_order' => 900,
					'user_defined' => false,
					'default' => '',
					'searchable' => false,
					'filterable' => false,
					'comparable' => false,
					'visible_on_front' => false,
					'used_in_product_listing' => true,
					'unique' => false,
					'apply_to' => ''
				]
			);
				$eavSetup->addAttribute(
					\Magento\Catalog\Model\Product::ENTITY,
					'upc',
					[
						'type' => 'text',
						'backend' => '',
						'frontend' => '',
						'label' => 'UPC Code',
						'input' => 'text',
						'class' => '',
						'source' => '',
						'global' => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
						'visible' => true,
						'required' => false,
						'sort_order' => 900,
						'user_defined' => false,
						'default' => '',
						'searchable' => false,
						'filterable' => false,
						'comparable' => false,
						'visible_on_front' => false,
						'used_in_product_listing' => true,
						'unique' => false,
						'apply_to' => '',
					]);
					$eavSetup->addAttribute(
						\Magento\Catalog\Model\Product::ENTITY,
						'backend_upc',
						[
							'type' => 'text',
							'backend' => '',
							'frontend' => '',
							'label' => 'UPC',
							'input' => 'text',
							'class' => '',
							'source' => '',
							'global' => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
							'visible' => false,
							'required' => false,
							'sort_order' => 900,
							'user_defined' => false,
							'default' => '',
							'searchable' => false,
							'filterable' => false,
							'comparable' => false,
							'visible_on_front' => false,
							'used_in_product_listing' => true,
							'unique' => false,
							'apply_to' => '',
						]);
					}
		if (version_compare($context->getVersion(), '1.2.3') < 0) {
	        $customerSetup = $this->customerSetupFactory->create(['setup' => $setup]);

	        $setup->startSetup();

	        /**
	         * stripe_customer_id customer attribute: Stores Stripe customer ID for each customer.
	         */
	        $customerSetup->addAttribute(
	            \Magento\Customer\Model\Customer::ENTITY,
	            'quivers_stripe_customer_id',
	            [
	                'label'            => 'Quivers Stripe Customer ID',
	                'type'             => 'varchar',
	                'input'            => 'text',
	                'default'          => '',
	                'position'         => 70,
	                'visible'          => false,
	                'required'         => false,
	                'system'           => false,
	                'user_defined'     => true,
	                'visible_on_front' => false,
	            ]
	        );

	        $profileIdAttr = $customerSetup->getEavConfig()->getAttribute(
	            \Magento\Customer\Model\Customer::ENTITY,
	            'quivers_stripe_customer_id'
	        );

	        $profileIdAttr->addData([
	            'attribute_set_id' => $customerSetup->getDefaultAttributeSetId(\Magento\Customer\Model\Customer::ENTITY),
	            'attribute_group_id' => $customerSetup->getDefaultAttributeGroupId(\Magento\Customer\Model\Customer::ENTITY),
	            'used_in_forms' => [],
	        ]);

	        $this->attributeRepository->save($profileIdAttr);

	        $setup->endSetup();
	    }
	    if (version_compare($context->getVersion(), '1.2.4') < 0) {
	        $customerSetup = $this->customerSetupFactory->create(['setup' => $setup]);

	        $setup->startSetup();

	        /**
	         * stripe_customer_cardIds customer attribute: Stores Stripe CardIds for each customer.
	         */
	        $customerSetup->addAttribute(
	            \Magento\Customer\Model\Customer::ENTITY,
	            'quivers_stripe_card_ids',
	            [
	                'label'            => 'Quivers Stripe Cards Ids',
	                'type'             => 'varchar',
	                'input'            => 'text',
	                'default'          => '',
	                'position'         => 80,
	                'visible'          => false,
	                'required'         => false,
	                'system'           => false,
	                'user_defined'     => true,
	                'visible_on_front' => false,
	            ]
	        );

	        $profileIdAttr = $customerSetup->getEavConfig()->getAttribute(
	            \Magento\Customer\Model\Customer::ENTITY,
	            'quivers_stripe_card_ids'
	        );

	        $profileIdAttr->addData([
	            'attribute_set_id' => $customerSetup->getDefaultAttributeSetId(\Magento\Customer\Model\Customer::ENTITY),
	            'attribute_group_id' => $customerSetup->getDefaultAttributeGroupId(\Magento\Customer\Model\Customer::ENTITY),
	            'used_in_forms' => [],
	        ]);

	        $this->attributeRepository->save($profileIdAttr);

	        $setup->endSetup();
	    }
    }
}